#!/usr/bin/env python3

import socketserver
import signal
import time
import os

from base64 import b64decode
from base64 import b64encode

from secrets import FLAG, A, B, C

IP_ADDR = "0.0.0.0"
PORT = 4444

# ------------------------------------------------------

class LCG(object):
    def __init__(self, seed, a, b, c):
        self.x = seed % c
        self.a = a
        self.b = b
        self.c = c

    def next(self):
        self.x = (self.x * self.a + self.b) % self.c
        return self.x

def wrap(data):
    return b64encode(data)

def unwrap(data):
    return b64decode(data)

# ------------------------------------------------------
    
def challenge(req):
    global FLAG, A, B, C

    seed = int.from_bytes(os.urandom(32), 'big')
    prng = LCG(seed, A, B, C)
    guessed = 0    
    
    req.sendall(b'Welcome to LCG-Lottery.\n\n')
    time.sleep(1)
    
    while True:
        try:
            req.sendall(b'Make a choice:\n' +\
                        b' [1] Draw\n' +\
                        b' [2] Guess\n\n')
            opt = req.recv(4096).decode().strip()
            if opt == "1":
                number = prng.next()
                print(number)
                data_out = wrap(str(number).encode("utf-8")) + b'\n'
                req.sendall(data_out)
            elif opt == "2":
                data_in = req.recv(4096).decode().strip()
                try:
                    guess = unwrap(data_in)
                    if int(guess) == prng.next():
                        guessed += 1
                        if guessed > 3:
                            data_out = b'Thats it: ' + FLAG + b'\n'
                        else:
                            data_out = b'Keep going!\n'
                    else:
                        data_out = b'Wrong!\n'
                        guessed = 0
                    req.sendall(data_out)
                except Exception as e:
                    req.sendall(b'[ERROR]: Could not understand data: make sure to base64 your payload!\n')
                    print(e)
                    continue
            else:
                req.sendall(b'[ERROR]: Baaad choice\n')

        except Exception as e:
            print(e)
            exit(1)
        
class incoming(socketserver.BaseRequestHandler):
    def handle(self):
        signal.alarm(300)
        while True:
            challenge(self.request)

class ReusableTCPServer(socketserver.ForkingMixIn, socketserver.TCPServer):
    pass

# ------------------------------------------------------

def main():
    global IP_ADDR, PORT
    socketserver.TCPServer.allow_reuse_address = True
    server = ReusableTCPServer((IP_ADDR, PORT), incoming)
    print(f"Listening on ({IP_ADDR}, {PORT})")
    server.serve_forever()

if __name__ == "__main__":    
    main()
